<?php
/*******************************************************************
 * This is the main class for all EM Addons by Stonehenge Creations.
 *******************************************************************/

// Exit if accessed directly
if (!defined('ABSPATH')) exit;
include_once(ABSPATH.'wp-admin/includes/plugin.php');



#===============================================
# 	Define the Core of all Addons.
#===============================================
if( !class_exists('EM_Addon_Base') ) {
	Class EM_Addon_Base {

		public function __construct() {
			add_action('admin_menu', array($this, 'create_admin_menu'), 10, 2);
			include('more.php');
		}

		public static function create_main_plugin_data() {
			$main = array(
				'name' 		=> 'Stonehenge EM Add-ons',
				'short' 	=> 'EM Add-ons',
				'text' 		=> 'stonehenge-em-addons',
				'slug' 		=> 'stonehenge_em_addons',
				'capab'		=> 'manage_options',
				'parent' 	=> 'stonehenge_em_addons',
				'icon' 		=> 'dashicons-calendar',
			);
			return $main;
		}

		public static function create_admin_menu() {
			$main 	= self::create_main_plugin_data();
			add_menu_page( $main['name'], $main['short'], $main['capab'], $main['slug'], array($this, 'show_main_page'), $main['icon'], 26);
			add_submenu_page( $main['slug'], $main['name'], $main['short'], $main['capab'], $main['slug'], array($this, 'show_main_page'));
			remove_submenu_page($main['slug'], $main['slug']);
			do_action('em_addons_menu');
		}


		public static function show_main_page() {
			/* Nothing to display */
		}

	} // End class.

	new EM_Addon_Base();
}



#===============================================
# 	Create a new EM Addon Plugin.
#===============================================
if( !class_exists('EM_Addon') ) {
	Class EM_Addon {

		public function __construct($addon) {

			// Create fallbacks.
			$addon['text'] 	= isset($addon['text']) ? $addon['text'] : str_replace('_', '-', $addon['slug']);
			$addon['slug'] 	= isset($addon['slug']) ? $addon['slug'] : str_replace('_', '-', $addon['text']);
			$addon['icon'] 	= isset($addon['icon']) ? $addon['icon'].'&nbsp;' : '';

			ksort($addon);
			$this->addon = $addon;
			global $config;
			$config = $addon;

			add_action('admin_init', array($this, 'setup_this_addon'));
			add_action('admin_init', array($this, 'register_option_fields'));
			add_action('em_addons_menu', array($this, 'create_sub_menu'), $addon['prio']);
			add_filter('plugin_action_links', array($this, 'add_settings_link'), 10, 2);
			add_filter('plugin_row_meta', array($this, 'create_additional_plugin_links'), 10, 2);
		}


		#===============================================
		# 	Start using this Addon.
		#===============================================
		public static function setup_this_addon() {
			$addon 	= $this->addon;
			$check 	= EM_Addon::check_version($addon);
			$saved 	= get_option( $addon['slug'] );

			// Is this a newer version?
			if( $check ) {
				return;
			}

			// Is this a new install?
			if( !$saved || empty($saved) || !is_array($saved) ) {
				// Create & set default values.
				EM_Addon::set_defaults($addon);
				EM_Addon::show_settings_notice($addon, 'error');
			}
			else {
				if( $saved && empty($saved) && is_array($saved) && method_exists( $addon['class'], 'update_this_plugin') ) {
					$addon['class']::update_this_plugin();
				}
			}
			// We're all done. Prevent loop.
			EM_Addon::update_version($addon);
			return;
		}


		#===============================================
		# 	Save new Defaults to Database.
		#===============================================
		public static function set_defaults($addon) {
			// Already checked if the database has values.
			$defaults 	= EM_Addon::create_defaults($addon);
			if( !add_option( $addon['slug'], $defaults, '', 'no') ) {
				update_option( $addon['slug'], $defaults);
			}
			return;
		}


		#===============================================
		# 	Create a full array with default values.
		#===============================================
		public static function create_defaults($addon) {
			$defaults 	= array();
			$sections 	= $addon['class']::define_option_fields();

			foreach($sections as $section) {
				foreach($section['fields'] as $field) {
					$id 		= $section['id'];
					$option 	= $field['id'];
					$no_fields 	= array('info', 'intro', 'span', 'notice', 'recipients', 'pro');
					if( !in_array( $field['type'], $no_fields) ) {
						$value = !empty($field['default']) ? $field['default'] : null;
						$defaults[$section['id']][$field['id']] = esc_attr($value);
					}
				}
			}
			if( count($defaults) == 1 ) {
				$defaults = array_values($defaults)[0];
			}

			if( method_exists( $addon['class'], 'add_defaults') ) {
				$extra 		= $addon['class']::add_defaults();
				$defaults 	= wp_parse_args_recursive($extra, $defaults);
			}
			return $defaults;
		}


		#===============================================
		# 	Check Addon Version Number.
		#===============================================
		public static function check_version( $addon ) {
			$version_slug 	= esc_html( $addon['slug'] . '_version' );
			$old_version 	= esc_html( get_option( $version_slug ) );
			$new_version 	= esc_html( $addon['version'] );
			$result 		= ($old_version != $new_version) ? false : true;
			return $result;
		}


		#===============================================
		# 	Save the current Addon Version to the Database.
		#===============================================
		public static function update_version($addon) {
			$version_slug 	= esc_html( $addon['slug'] . '_version' );
			$old_version 	= esc_html( get_option( $version_slug ) );
			$new_version 	= esc_html( $addon['version'] );

			// Only update if needed.
			if( $old_version != $new_version ) {
				if( !add_option( $version_slug, $new_version, '', 'no') ) {
					update_option( $version_slug, $new_version );
				}
			}
			return;
		}


		#===============================================
		# 	Create Sub Menu Item for this Addon.
		#===============================================
		public function create_sub_menu() {
			$addon 		= $this->addon;
			$sub_page 	= add_submenu_page(
				'stonehenge_em_addons',
				__($addon['name'], $addon['text']),
				$addon['icon'] . esc_html__($addon['short'], $addon['text']),
				'manage_options',
				esc_html($addon['slug']),
				array($this, 'show_options_page')
			);
			add_action('admin_print_styles-'.$sub_page, array($addon['class'], 'load_addon_assets'));
		}


		#===============================================
		# 	Render the Options Page for this Addon.
		#===============================================
		public static function show_options_page() {
			$addon 		= $this->addon;
			$text 		= $addon['text'];
			$sections 	= $addon['class']::define_option_fields();

			?>
			<div class="wrap">
				<h1><?php echo wp_kses_allowed($addon['icon'] . __( $addon['name'], $text) ); ?></h1>
				<?php
				if (isset($_GET['settings-updated'])) {
					echo '<div><p class="stonehenge-success big" id="remove-me">'. esc_html__('Your settings have been saved.', $text) .'</p></div>';
				}
				if( isset($addon['intro']) ) {
					echo '<p>' . wp_kses_allowed( __($addon['intro'], $text) ) .'</p>';
				}
				?>
				<div class="listFieldError" style="display:none;"></div>
				<div id="poststuff">
					<div class="inside">
						<div id="postbox-container" class="postbox-container stonehenge-settings">
							<div class="meta-box-sortables ui-sortable" id="normal-sortables">
								<form method="post" action="options.php" id="stonehenge-form" autocomplete="off" data-parsley-validate="" novalidate="">
									<?php
										settings_fields($addon['slug']);
										do_settings_sections($addon['slug']);
										self::display_option_fields( $addon );
										do_action('em_addons_show_extra');
									?>
									<div class="listFieldError" style="display:none;"></div>
									<input type="submit" class="stonehenge-button" value="<?php esc_html_e('Save Changes'); ?>" onclick="setTimeout(showErrors, 100)">
								</form>
							</div>
						</div>
					</div>
				</div>
			</div>
			<?php
		}


		#===============================================
		# 	Make Plugin Options usable.
		#===============================================
		public static function display_option_fields( $addon ) {
			$saved 		= get_option( $addon['slug'] );
			$options	= $addon['class']::define_option_fields();

			if( !is_array($options) ) {
				return;
			}

			$closed 	= !$saved || empty($saved) || (count($options) < 3) ? '' : 'closed';
			foreach( $options as $option ) {
				?>
				<div class="postbox stonehenge-metabox <?php echo $closed; ?>" id="<?php echo esc_html($option['id']); ?>">
					<h3 class="hndle"><?php esc_html_e($option['label'], $addon['text']); ?></h3>
					<div class="inside">
						<table class="stonehenge-table">
						<?php
						foreach( $option['fields'] as $fields => $field ) {
							$default 	= isset($field['default']) ? wp_kses_allowed($field['default']) : '';
							if( count($options) > 1 ) {
								$tab_id		= esc_attr($option['id'], ENT_QUOTES);
								$field_id	= esc_attr($field['id'], ENT_QUOTES);
								$id 		= esc_attr($option['id'].'_'. $field['id'], ENT_QUOTES);
								$type 		= esc_attr($field['type'], ENT_QUOTES);
								$name 		= "{$addon['slug']}[{$option['id']}][{$field['id']}]";
								$name 		= esc_attr($name, ENT_QUOTES);
								$class 		= str_replace('_', '-', "{$option['id']}-{$field['id']}");
								$class 		= esc_attr($class, ENT_QUOTES);
								$value 		= isset($saved[$option['id']][$field['id']]) ? $saved[$option['id']][$field['id']] : $default;
								$checked 	= isset($saved[$option['id']][$field['id']]) ? 'checked="checked"' : '';
							} else {
								$id 		= esc_attr($field['id'], ENT_QUOTES);
								$type 		= esc_attr($field['type'], ENT_QUOTES);
								$name 		= "{$addon['slug']}[{$field['id']}]";
								$name 		= esc_attr($name, ENT_QUOTES);
								$class 		= str_replace('_', '-', "{$field['id']}");
								$class 		= esc_attr($class, ENT_QUOTES);
								$value 		= isset($saved[$field['id']]) ? $saved[$field['id']] : $default;
								$checked 	= isset($saved[$field['id']]) ? 'checked="checked"' : '';
							}
							$size 		= isset($field['size']) ? $field['size'] : 'large-text';
							$size 		= esc_attr($size, ENT_QUOTES);
							$min 		= isset($field['min']) ? $field['min'] : '1';
							$min 		= esc_attr($min, ENT_QUOTES);
							$max 		= isset($field['max']) ? $field['max'] : '9999';
							$max 		= esc_attr($max, ENT_QUOTES);
							$helper		= isset($field['helper']) ? '<p class="description">'. wp_kses_allowed($field['helper']) .'</p>' : '';
							$before	 	= isset($field['before']) ? '<p>'. wp_kses_allowed($field['before']) .'</p>' : '';
							$after		= isset($field['after']) ? '&nbsp;&nbsp;'. $field['after'] : '';
							$required	= isset($field['required']) && ($field['required'] != false) ? esc_attr('required=required', ENT_QUOTES) : '';
							$choices 	= isset($field['choices']) ? $field['choices'] : array('no' => __('No'), 'yes' => __('Yes')) ;
							$label 		= "<th scope='row'><label for='{$id}'>". ($field['label']) ."</label></th><td>{$before}";

							echo "<tr class='{$class}'>";
							switch( $field['type'] ) {
								case 'info':
								case 'intro':
								case 'notice':
									echo '<td colspan="2"><p>'. wp_kses_allowed($default) .'</p>';
									break;
								case 'span':
									echo $label . wp_kses_allowed($default);
									break;
								case 'text':
								case 'email':
								case 'url':
								case 'password':
									echo sprintf('%s<input type="%s" id="%s" name="%s" value="%s" class="%s" %s>',
										$label, $type, $id, $name, esc_attr($value, ENT_QUOTES), $size, $required);
									break;
								case 'number':
									echo sprintf('%s<input type="%s" id="%s" name="%s" value="%s" class="small" %s min="%s" max="%s">',
										$label, $type, $id, $name, esc_attr($value, ENT_QUOTES), $required, $min, $max);
									break;
								case 'select':
								case 'dropdown':
									echo "{$label}<select name='{$name}' id='{$id}' {$required}>";
									// Add an empty option first.
									echo sprintf('<option value="" disabled selected>- %s -</option>', esc_html__('Select') );
									foreach( $choices as $k => $v ) {
										$selected = ($k === $value ? 'selected' : '');
										echo sprintf('<option value="%s" %s>%s</option>', esc_attr($k, ENT_QUOTES), $selected, esc_html($v) );
									}
									echo '</select>';
									break;
								case 'textarea':
									echo sprintf('%s<textarea id="%s" name="%s" rows="10" %s>%s</textarea>', $label, $id, $name, $required, wp_kses_allowed(stripslashes($value)) );
									break;
								case 'editor':
									echo "{$label}";
									if( isset($field['required']) && ($field['required'] != false) ? add_filter('the_editor', array('EM_Addon', 'require_editor'), 10, 1) : '');
									$args = array(
									    'wpautop' => true,
									    'media_buttons' => false,
									    'textarea_name' => $name,
									    'textarea_rows' => 10,
									    'editor_css' => '',
									    'editor_class' => '',
									    'teeny' => false,
									    'dfw' => true,
									    'tinymce' => false, // <-----
									    'quicktags' => true
									);
									wp_editor( wp_kses_allowed(stripslashes($value)), $id, $args );
									if( isset($field['required']) && ($field['required'] != false) ? remove_filter('the_editor', array('EM_Addon', 'require_editor'), 10, 1): '');
									break;
								case 'file':
									wp_enqueue_script( 'jquery' );
									wp_enqueue_media();
									$file 	= esc_html__('Choose file', $addon['text']);
									$remove = esc_html__('Clear');
									$input 	= sprintf('<input type="url" id="%s" name="%s" value="%s" class="%s filename" readonly>',
										$id, $name, esc_url_raw($value, ENT_QUOTES), $size);
									$select = sprintf('<button type="button" id="%s_button" class="button-secondary" title="%s">%s</button>',
										$id, esc_attr($file, ENT_QUOTES), '&#x1F4CE; '. esc_html($file) );
									$clear 	= sprintf('<button type="button" class="button-secondary clear-file" title="%s">%s</button>',
										esc_attr($remove, ENT_QUOTES), '&#x2718;');
									echo sprintf('%s%s&nbsp;&nbsp;%s&nbsp;&nbsp;%s',
										$label, $input, $select, $clear
									);
									break;
								case 'checkboxes':
									echo "{$label}<div id='checkboxes' class='{$class}'>";
									foreach( $choices as $key => $result) {
										if(@$saved[$tab_id][$field_id] && in_array($key, $saved[$tab_id][$field_id])) { $checked = esc_attr('checked=checked', ENT_QUOTES); }
										else { $checked = null; }
										echo sprintf( '<input type="checkbox" id="%s" name="%s" value="%s" %s %s> <label for="%s">%s</label><br>',
											esc_attr($tab_id.'_'.$key, ENT_QUOTES), $name.'[]', esc_attr($key, ENT_QUOTES), $checked, 'data-parsley-group="status" '.$required, esc_attr($tab_id.'_'.$key, ENT_QUOTES), esc_html($result) );
									}
									echo '</div>';
									break;

								case 'toggle':
									echo $label . '<div class="switch-toggle switch-holo" style="width: 120px; height:1.85em;">';
									foreach ( $choices as $v => $l ) {
										$checked = $v === $value ? esc_attr('checked=checked', ENT_QUOTES) : '';
										echo sprintf( '<input type="radio" name="%s" id="%s" value="%s" %s %s><label for="%s">%s</label>',
											$name, esc_attr($id.'_'.$v, ENT_QUOTES), esc_attr($v, ENT_QUOTES), $checked, $required, esc_attr($id.'_'.$v, ENT_QUOTES), esc_html($l)
										);
									}
									echo '<a></a></div>';
									break;
									case 'switch':
										echo sprintf('%s<label class="switch"><input type="checkbox" id="%s" name="%s" value="yes" %s><span data-unchecked="%s" data-checked="%s"></span></label>',
											$label, $id, $name, $checked, esc_attr( __('No'), ENT_QUOTES), esc_attr( __('Yes'), ENT_QUOTES));
									break;
								case 'radio':
									echo sprintf('%s<fieldset><legend class="screen-reader-text"></legend>', $label);
									$c = 0;
									foreach( $choices as $v => $l ) {
										$checked = $v === $value ? esc_attr('checked=checked', ENT_QUOTES) : '';
										echo sprintf('<label><input type="radio" name="%s" id="%s" value="%s" %s %s><span>%s</span></label>%s',
											$name, esc_attr($id.'_'.$v, ENT_QUOTES), esc_attr($v, ENT_QUOTES), $checked, $required, esc_html($l), ($c < count($choices) - 1 ? '<br>' : ''));
										$c++;
									}
									echo "</fieldset>";
									break;
								case 'color':
									wp_enqueue_style('farbtastic');
									wp_enqueue_script('farbtastic');
									echo sprintf('%s<div style="position:relative;"><input type="text" name="%s" id="%s" %s><input type="button" class="pickcolor button-secondary" value="%s"><div class="colorpicker" style="z-index:100; position:absolute; display:none;"></div></div>',
									$label, $name, $id, $required, esc_html__('Select Color') );
									break;

								case 'date':
									wp_enqueue_script('jquery-ui-datepicker');
									wp_enqueue_style('datepicker', plugins_url('datepicker.min.css', __FILE__));
									echo sprintf('%s<input type="text" name="%s" value="%s" class="datepicker" autocomplete="off" size="21" %s>',
										$label, $name, esc_attr($value, ENT_QUOTES), $required);
									break;

								case 'time':
									wp_enqueue_script('timepicker-js', plugins_url('timepicker.min.js', __FILE__), '', array('jquery'), false);
									echo sprintf('%s<input type="text" id="%s" name="%s" value="%s" class="time-picker" size="10" autocomplete="off">',
										$label, $id, $name, esc_attr($value, ENT_QUOTES) );
									break;
								default:
									echo "{$label}Field not defined yet: {$type}</td>";
									break;
							}
							echo wp_kses_allowed($after) . wp_kses_allowed($helper) .'</td></tr>';
							}
						?>
						</table>
					</div>
				</div>
				<?php
			}
		}


		#===============================================
		# 	Register settings in WordPress options.php
		#===============================================
		public static function register_option_fields() {
			$addon 	= $this->addon;
			register_setting( $addon['slug'], $addon['slug'], array( $addon['class'], 'sanitize_option_fields') );
		}


		#===============================================
		# 	Sanitize Form Inputs.
		#===============================================
		public static function sanitize_option_fields($input, $sections) {
			if( count($sections) > 1 ) {
				$clean = EM_Addon::sanitize_multiple( $input, $sections );
			} else {
				$clean = EM_Addon::sanitize_single( $input, $sections );
			}
			return $clean;
		}


		#===============================================
		# 	Sanitize Form Input for mutiple sections.
		#===============================================
		public static function sanitize_multiple( $input, $sections ) {
			$clean = array();

			// Sanitize by form field input type.
			foreach($sections as $section) {
				foreach($section['fields'] as $fields => $field) {
					$id 	= "{$section['id']}_{$field['id']}";
					$type 	= $field['type'];
					foreach($input as $tabs => $tab) {
						foreach($tab as $key => $value) {
							$tab_key = $tabs.'_'.$key;
							switch($field['type']) {
								// IMPORTANT: Do not use 'default' -> it will process all again.
								case 'text':
								case 'number':
								case 'tel':
								case 'phone':
								case 'password':
								case 'select':
								case 'dropdown':
								case 'color':
									if($id === $tab_key) { $clean[$tabs][$key] = sanitize_text_field(stripslashes($input[$tabs][$key])); }
									break;
								case 'time':
									if($id === $tab_key) { $clean[$tabs][$key] = sanitize_text_field(stripslashes(date("H:i:s",strtotime($input[$tabs][$key])))); }
								break;
								case 'date':
									if($id === $tab_key) { $clean[$tabs][$key] = sanitize_text_field(stripslashes(date("Y-m-d",strtotime($input[$tabs][$key])))); }
								break;
								case 'toggle':
								case 'switch':
								case 'radio':
									if($id === $tab_key) { $clean[$tabs][$key] = sanitize_text_field(wp_unslash($input[$tabs][$key])); }
									break;
								case 'email':
									if($id === $tab_key) { $clean[$tabs][$key] = strtolower(sanitize_email($input[$tabs][$key])); }
									break;
								case 'textarea':
									if($id === $tab_key) { $clean[$tabs][$key] = wp_kses_allowed(stripslashes($input[$tabs][$key])); }
									break;
								case 'editor':
									if($id === $tab_key) { $clean[$tabs][$key] = wp_kses_allowed(stripslashes($input[$tabs][$key])); }
									break;
								case 'url':
								case 'media':
								case 'image':
								case 'file':
									if($id === $tab_key) { $clean[$tabs][$key] = esc_url_raw($input[$tabs][$key]); }
									break;
								case 'checkbox':
									if($id === $tab_key) { $clean[$tabs][$key] = sanitize_key($input[$tabs][$key]); }
									break;
								case 'checkboxes':
									if($id === $tab_key) { $clean[$tabs][$key] = @array_map('sanitize_text_field', wp_unslash($input[$tabs][$key])); }
									break;
								case 'multi':
								case 'select_multi':
									if($id === $tab_key) { $clean[$tabs][$key] = array_map('sanitize_text_field', $input[$tabs][$key]); }
									break;
							}
						}
					}
				}
			}
			return $clean;
		}


		#===============================================
		# 	Sanitize Form Input for a single section.
		#===============================================
		public static function sanitize_single( $input, $sections ) {
			$clean = array();
			// Sanitize by form field input type.
			foreach( $sections[0]['fields'] as $fields => $field ) {
				$id = $field['id'];
				foreach( $input as $key => $value ) {
					switch($field['type']) {
						// IMPORTANT: Do not use 'default' -> it will process all again.
						case 'text':
						case 'number':
						case 'tel':
						case 'phone':
						case 'password':
						case 'select':
						case 'dropdown':
						case 'color':
							if($id === $key) { $clean[$key] = sanitize_text_field(stripslashes($input[$key])); }
							break;
						case 'time':
							if($id === $key) { $clean[$key] = sanitize_text_field(stripslashes(date("H:i:s",strtotime($input[$key])))); }
						break;
						case 'date':
							if($id === $key) { $clean[$key] = sanitize_text_field(stripslashes(date("Y-m-d",strtotime($input[$key])))); }
						break;
						case 'toggle':
						case 'switch':
						case 'radio':
							if($id === $key) { $clean[$key] = sanitize_text_field(wp_unslash($input[$key])); }
							break;
						case 'email':
							if($id === $key) { $clean[$key] = strtolower(sanitize_email($input[$key])); }
							break;
						case 'textarea':
						case 'editor':
							if($id === $key) { $clean[$key] = wp_kses_allowed(stripslashes($input[$key])); }
							break;
						case 'url':
						case 'media':
						case 'image':
						case 'file':
							if($id === $key) { $clean[$key] = esc_url_raw($input[$key]); }
							break;
						case 'checkbox':
							if($id === $key) { $clean[$key] = sanitize_key($input[$key]); }
							break;
						case 'checkboxes':
							if($id === $key) { $clean[$key] = @array_map('sanitize_text_field', wp_unslash($input[$key])); }
							break;
						case 'multi':
						case 'select_multi':
							if($id === $key) { $clean[$key] = array_map('sanitize_text_field', $input[$key]); }
							break;
					} // End switch.
				}
			}
			return $clean;
		}


		#===============================================
		# 	Enqueue Scripts & Styles for Options Page.
		#===============================================
		public static function load_main_assets() {
			wp_register_style('stonehenge-css', plugins_url('stonehenge.css', __FILE__) );
			wp_register_script('stonehenge-js', plugins_url('stonehenge.js', __FILE__), array('jquery'), '1.0', true);
			wp_localize_script('stonehenge-js', 'CORE', self::localize_js_strings());

			// Parsley validation.
			$locale 	= substr(get_bloginfo('language'), 0, 2);
			wp_register_script('parsley-validation', plugins_url('parsley/parsley.min.js', __FILE__), null, '1.0', true);
			wp_register_script('parsley-locale', plugins_url('parsley/i18n/'. $locale .'.js', __FILE__), null, '1.0', true);
			wp_register_script('parsley-locale-extra', plugins_url('parsley/i18n/'. $locale .'.extra.js', __FILE__), null, '1.0', true);

			$styles 	= array('stonehenge-css');
			$scripts 	= array('stonehenge-js', 'parsley-validation', 'parsley-locale', 'parsley-locale-extra');

			wp_enqueue_style( $styles );
			wp_enqueue_script( $scripts );
		}


		#===============================================
		# 	Make JavaScript strings translatable.
		#===============================================
		public static function localize_js_strings() {
			global $config;
			$text 			= $config['text'];
			$saved 			= get_option( $config['slug'] );
			$date_form_js 	= get_option('dbem_date_format_js');
			$date_js 		= isset( $date_form_js ) ? $date_form_js : 'yy-mm-dd';
			$time_format 	= get_option('time_format');
			$localize 		= array(
				'add' => __('Add'),
				'remove' => __('Remove'),
				'edit' => __('Edit'),
				'choosefile' => __('Choose File'),
				'showError' => __('Not all required fields were filled out. Please check all sections and try again.', $text),
				'year_range' => '"'. date('Y') .':'. (date('Y') + 5) .'"',
				'date_format_js' => $date_js,
				'timeFormat' => $time_format,
			);
			return $localize;
		}


		#===============================================
		# 	Add "Settings" link to WP Plugins Page.
		#===============================================
		function add_settings_link( $links, $file ) {
			$config = $this->addon;
			$name 	= $config['text'];

			// Remove links if Pro version is activated.
			if( $file === 'events-manager-email-users/events-manager-email-users.php' && class_exists('EM_Email_Users_Pro')) {
				return;
			}

			if( $file != plugin_basename("{$name}/{$name}.php")) {
				return $links;
			}
			else {
				$settings_link = sprintf( '<a href="%s">%s</a>', $config['url'], __('Settings') );
				array_unshift($links, $settings_link);
				return $links;
			}
		}


		#===============================================
		# 	Add additional links to WP Plugins Page.
		#===============================================
		function create_additional_plugin_links( $links, $file ) {
			$config 		= $this->addon;
			$text 			= $config['text'];
			$author			= 'DuisterDenHaag';
			$donate			= 'https://useplink.com/payment/VRR7Ty32FJ5mSJe8nFSx';
			$support_link 	= array('<a href="https://wordpress.org/support/plugin/'.$text.'" target="_blank">'.__('Support').'</a>');
			$donate_link  	= array('&#x2665; <a href="'.$donate.'" target="_blank">'.__('Donate', $text).'</a>');

			$rate_url 		= 'https://wordpress.org/support/plugin/'. $text .'/reviews/?rate=5#new-post';
			$rate_link 		= array('&#x1F31F; <a href="'.$rate_url.'" target="_blank">'. __('Rate this plugin', $text) .'</a>' );

			// Only target this plugin.
			if( $file != plugin_basename( "{$text}/{$text}.php")) {
				return $links;
			}

			// Remove links if Pro version is activated.
			if( $file === 'events-manager-email-users/events-manager-email-users.php' && class_exists('EM_Email_Users_Pro')) {
				return array_merge($links, $rate_link);
			}

			return array_merge($links, $support_link, $donate_link, $rate_link);
		}


		#===============================================
		# 	Create a notice message.
		#===============================================
		public static function show_notice($notice, $class) {
			$notice  = wp_kses_allowed( $notice );
			return( '<p><span class="stonehenge-'.$class.'">'. $notice .'</span></p>' );
		}


		#===============================================
		# 	Show an Admin Notice if needed.
		#===============================================
		public static function show_settings_notice($config, $class) {
			if( !current_user_can('manage_options') ) {
				return;
			}
			$message = sprintf("<strong>{$config['short']}: </strong>" . _x('Please check your <a href=%1$s>settings</a> and click on "%2$s".', 'Please check your settings and click on "Save Changes".', $config['text']), $config['url'], __('Save Changes') );

			add_action('admin_notices', function () use ($class, $message) {
				printf('<div class="notice notice-%1$s"><p>%2$s</p></div>', esc_html($class), $message);
			});
		}


		#===============================================
		# 	Show an Admin Notice if new Plugin Options have been added.
		#===============================================
		public static function show_new_options_notice($config, $class) {
			if( !current_user_can('manage_options') ) {
				return;
			}
			$message = sprintf('<strong>%2$s: </strong>' . __('New options have been added. Please, check your <a href=%1$s>settings</a>.', $config['text']), $config['url'], $config['short'] );
			$message = wp_kses_allowed( $message );
			add_action('admin_notices', function () use ($class, $message) {
				printf('<div class="notice notice-%1$s"><p>%2$s</p></div>', esc_html($class), $message);
			});
		}


		#===============================================
		# 	Minify JS.
		#===============================================
		public static function minify_js($input) {
		    if(trim($input) === "") return $input;
		    return preg_replace( array('#\s*("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\')\s*|\s*\/\*(?!\!|@cc_on)(?>[\s\S]*?\*\/)\s*|\s*(?<![\:\=])\/\/.*(?=[\n\r]|$)|^\s*|\s*$#', '#("(?:[^"\\\]++|\\\.)*+"|\'(?:[^\'\\\\]++|\\\.)*+\'|\/\*(?>.*?\*\/)|\/(?!\/)[^\n\r]*?\/(?=[\s.,;]|[gimuy]|$))|\s*([!%&*\(\)\-=+\[\]\{\}|;:,.<>?\/])\s*#s', '#;+\}#', '#([\{,])([\'])(\d+|[a-z_][a-z0-9_]*)\2(?=\:)#i', '#([a-z0-9_\)\]])\[([\'"])([a-z_][a-z0-9_]*)\2\]#i'), array( '$1', '$1$2', '}', '$1$3', '$1.$3'), $input);
		}


		#===============================================
		# 	Minify CSS.
		#===============================================
		public static function minify_css( $input ) {
			$output = preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $input);
			$output = str_replace(': ', ':', $output);
			$output = str_replace(array("\r\n", "\r", "\n", "\t", '  ', '    ', '    '), '', $output);
			$css 	= '<style>'. $output .'</style>';
			return $css;
		}


		#===============================================
		# 	Make wp_editor a required field.
		#===============================================
		public static function require_editor( $editor ) {
	    	$editor = str_replace( '<textarea', '<textarea required="required"', $editor );
			return $editor;
		}


		#===============================================
		# 	Convert Local Time to UTC.
		#===============================================
		public static function convert_to_UTC( $input, $format = false ) {
			if( !$format ) {
				$format 	= 'Y-m-d H:i:s';
			}
			$timezone 	= get_option('timezone_string');
			$local_tz 	= new DateTimeZone( $timezone );
			$UTC 		= new DateTimeZone( 'UTC' );
			$Date		= new DateTime( date( $format, strtotime( $input ) ), $local_tz );
			$Date->setTimezone( $UTC );
			$output 	= $Date->format( $format );
			return $output;
		}


		#===============================================
		# 	Convert UTC to Local TIme.
		#===============================================
		public static function convert_to_local( $input, $format = false ) {
			if( !$format ) {
				$format 	= 'Y-m-d H:i:s';
			}
			$timezone 	= get_option('timezone_string');
			$local_tz 	= new DateTimeZone( $timezone );
			$UTC 		= new DateTimeZone( 'UTC' );
			$Date		= new DateTime( date( $format, strtotime( $input ) ), $UTC );
			$Date->setTimezone( $local_tz );
			$output 	= $Date->format( $format );
			return $output;
		}

	} // End class.
}


if( !function_exists('wp_parse_args_recursive') ) {
	function wp_parse_args_recursive( &$a, $b ) {
		$a = (array) $a;
		$b = (array) $b;
		$result = $b;
		foreach ( $a as $k => &$v ) {
			if ( is_array( $v ) && isset( $result[$k] ) ) {
				$result[$k] = wp_parse_args_recursive( $v, $result[$k] );
			} else {
				$result[$k] = $v;
			}
		}
		return $result;
	}
}

if( !function_exists('wp_kses_allowed') ) {
	function wp_kses_allowed( $context ) {
		global $allowedposttags;
		return wp_kses( $context, $allowedposttags, wp_allowed_protocols() );
	}
}

